/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import org.eclipse.emf.common.command.CommandStack;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.interfaces.IStorageProvider;
import org.fortiss.tooling.kernel.services.ICommandStackService;
import org.fortiss.tooling.kernel.services.IPersistencyService;

/**
 * This class implements the {@link ICommandStackService} interface. This class
 * mainly delegates its calls to the {@link IPersistencyService}.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @levd.rating RED Rev:
 */
public class CommandStackService implements ICommandStackService {
	/** {@inheritDoc} */
	@Override
	public void runAsCommand(EObject target, Runnable runner) {
		IStorageProvider provider = IPersistencyService.INSTANCE
				.getStorageProviderFor(target);
		if (provider != null) {
			provider.runAsCommand(target, runner);
		}
	}

	/** {@inheritDoc} */
	@Override
	public boolean canUndo(EObject target) {
		CommandStack stack = getCommandStack(target);
		return stack != null && stack.canUndo();
	}

	/** {@inheritDoc} */
	@Override
	public boolean canRedo(EObject target) {
		CommandStack stack = getCommandStack(target);
		return stack != null && stack.canRedo();
	}

	/** {@inheritDoc} */
	@Override
	public void undo(EObject target) {
		getCommandStack(target).undo();
	}

	/** {@inheritDoc} */
	@Override
	public void redo(EObject target) {
		getCommandStack(target).redo();
	}

	/** Fetches command stack from persistency service. */
	private CommandStack getCommandStack(EObject target) {
		return IPersistencyService.INSTANCE.getCommandStack(target);
	}
}
