/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.conqat.ide.commons.ui.logging.LoggingUtils;
import org.conqat.lib.commons.collections.CollectionUtils;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.base.EObjectAwareServiceBase;
import org.fortiss.tooling.kernel.interfaces.IEditorBinding;
import org.fortiss.tooling.kernel.internal.editor.BindingEditor;
import org.fortiss.tooling.kernel.internal.editor.ModelElementEditorInput;
import org.fortiss.tooling.kernel.services.IEditorService;

/**
 * This class implements the {@link IEditorService} interface.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @levd.rating RED Rev:
 */
public class EditorService extends EObjectAwareServiceBase<IEditorBinding<EObject>>
		implements IEditorService {

	/** The compositor extension point ID. */
	private static final String EXTENSION_POINT_NAME = "org.fortiss.tooling.kernel.modelEditorBinding";

	/** The compositor configuration element name. */
	private static final String CONFIGURATION_ELEMENT_NAME = "modelEditorBinding";

	/** The compositor attribute name. */
	private static final String ATTRIBUTE_NAME = "binding";

	/** Constructor. */
	public EditorService() {
		super();
		// sort editor bindings by priority.
		EditorBindingComparator comparator = new EditorBindingComparator();
		for (Class<?> clazz : handlerMap.keySet()) {
			Collections.sort(handlerMap.get(clazz), comparator);
		}
	}

	/** {@inheritDoc} */
	@Override
	public void openInEditor(EObject element) {
		try {
			// FIXME (FH): only open editor, if at least one bound appear
			PlatformUI
					.getWorkbench()
					.getActiveWorkbenchWindow()
					.getActivePage()
					.openEditor(new ModelElementEditorInput(element),
							BindingEditor.ID);
		} catch (final PartInitException e) {
			LoggingUtils.error(ToolingKernelActivator.getDefault(),
					"Could not open editor with ID " + BindingEditor.ID, e);
		}
	}

	/** {@inheritDoc} */
	@Override
	public List<IEditorBinding<EObject>> getBindings(EObject element) {
		List<IEditorBinding<EObject>> bindings = getRegisteredHandlers(element
				.getClass());
		if (bindings == null) {
			bindings = CollectionUtils.emptyList();
		}
		return bindings;
	}

	/** {@inheritDoc} */
	@Override
	protected String getExtensionPointName() {
		return EXTENSION_POINT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getConfigurationElementName() {
		return CONFIGURATION_ELEMENT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getHandlerClassAttribute() {
		return ATTRIBUTE_NAME;
	}

	/** {@link Comparator} for {@link IEditorBinding}s. */
	private static class EditorBindingComparator implements
			Comparator<IEditorBinding<EObject>> {

		/** {@inheritDoc} */
		@Override
		public int compare(IEditorBinding<EObject> eb0,
				IEditorBinding<EObject> eb1) {
			return eb1.getPriority() - eb0.getPriority();
		}
	}
}
