/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.util;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.PlatformUI;

/**
 * Utility methods for {@link EObject} selections.
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @levd.rating RED Rev:
 */
public final class EObjectSelectionUtils {

	/**
	 * Returns the first selected element if it is an EObject or
	 * <code>null</code> otherwise.
	 */
	public static EObject getCurrentSelectionFirstElement() {
		return getFirstElement(getCurrentSelection());
	}

	/**
	 * Returns the list of selected elements filtered for EObjects.
	 */
	public static List<EObject> getCurrentSelectionEObjects() {
		return getEObjectElements(getCurrentSelection());
	}

	/**
	 * Returns the first selected EObject or <code>null</code> if no EObject is
	 * selected.
	 */
	public static EObject getFirstElement(ISelection selection) {
		if (selection instanceof IStructuredSelection && !selection.isEmpty()) {
			if (((IStructuredSelection) selection).getFirstElement() instanceof EObject) {
				return (EObject) ((IStructuredSelection) selection)
						.getFirstElement();
			}
		}
		return null;
	}

	/**
	 * Returns the current selection obtained from the platform.
	 */
	public static ISelection getCurrentSelection() {
		return PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getSelectionService().getSelection();
	}

	/** Extracts all {@link EObject}s from the given selection. */
	public static List<EObject> getEObjectElements(ISelection selection) {
		List<EObject> result = new ArrayList<EObject>();
		if (selection instanceof StructuredSelection) {
			for (Object o : ((StructuredSelection) selection).toList()) {
				if (o instanceof EObject) {
					result.add((EObject) o);
				}
			}
		}
		return result;
	}

}
